package am.lodge.platform.service.impl;


import am.lodge.platform.service.ExcelService;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.regex.Pattern;

/**
 * @ClassName: ExcelServiceImpl
 * @Description: 用于
 * Modification History:
 * Date                  Author                 Version       Description
 * ---------------------------------------------------------
 * 2018/9/3     ZhangShuling       v1.0.0
 */

@Service
public class ExcelServiceImpl implements ExcelService {

  private final String DATATYPE_INTEGER = "integer";
  private final String DATATYPE_FLOAT = "float";
  private final String DATATYPE_STRING = "string";
  private final String DATATYPE_DATE = "date";

  Map<String, HSSFCellStyle> cellStyles = null;

  public   List<List<Object>>  parse(InputStream in,int sheetIndex) throws Exception{
    Workbook wb =  WorkbookFactory.create(in);
    if(wb == null){
      return null;
    }
    return parse(wb,sheetIndex);
  }

  public   List<List<List<Object>>>  parse(InputStream in) throws Exception {
    List<List<List<Object>>> data = null;
    if(in==null){
      return data;
    }
    data = new ArrayList<List<List<Object>>>();
    Workbook wb =  WorkbookFactory.create(in);
    if(wb == null){
      return data;
    }
    for (int i = 0; i < wb.getNumberOfSheets(); i++) {
      data.add(parse(wb,i));
    }
    return data;
  }

  public void export(Map<String, Object> header, List<Map<String, Object>> data, String fileName, HttpServletResponse response) throws Exception {
    OutputStream out = response.getOutputStream();// 取得输出流
    response.reset();// 清空输出流
    String toFileName = fileName.endsWith("xls") || fileName.endsWith("xlsx") ? fileName : new StringBuilder(fileName)
      .append(".xls").toString();
    // 下面是对中文文件名的处理
    response.setCharacterEncoding("UTF-8");// 设置相应内容的编码格式
    toFileName = java.net.URLEncoder.encode(toFileName, "UTF-8");
    response.setHeader("Content-Disposition", "attachment;filename=" + new String(toFileName.getBytes("UTF-8"), "GBK"));
    response.setContentType("application/msexcel");// 定义输出类型
    export(header, data, fileName, out);
    out.close();
  }



  private Map<String, HSSFCellStyle> initCellStyles(HSSFWorkbook wb) {

    HSSFDataFormat format = wb.createDataFormat();

    Map<String, HSSFCellStyle> cellStyleMap = new HashMap<String, HSSFCellStyle>();
    HSSFCellStyle integerStyle = wb.createCellStyle();
    integerStyle.setAlignment(integerStyle.getAlignmentEnum());

    cellStyleMap.put(DATATYPE_INTEGER, integerStyle);

    HSSFCellStyle floatStyle = wb.createCellStyle();

    floatStyle.setDataFormat(format.getFormat("0.00")); // 两位小数
    cellStyleMap.put(DATATYPE_FLOAT, floatStyle);

    HSSFCellStyle stringStyle = wb.createCellStyle();

    cellStyleMap.put(DATATYPE_STRING, stringStyle);

    HSSFCellStyle dateStyle = wb.createCellStyle();
    dateStyle.setAlignment(dateStyle.getAlignmentEnum());
    cellStyleMap.put(DATATYPE_DATE, dateStyle);
    return cellStyleMap;
  }

  /**
   * 根据数据各数据类型，默认一下数据的格式样式
   *
   * @param wb
   * @param cell
   * @param value
   * @return
   */
  private void formatExcelCell(HSSFWorkbook wb, Cell cell, Object value) {
    if(value == null) return;
    if(cellStyles == null) cellStyles = initCellStyles(wb);

    HSSFCellStyle cellStyle = null;
    if(isInteger(value.toString())) {//  如果是整数，右对齐
        cellStyle = cellStyles.get(DATATYPE_INTEGER);
        cell.setCellValue(value.toString());
    } else if(isDouble(value.toString())) {//  如果是小数,保留2位小数，右对齐
        cellStyle = cellStyles.get(DATATYPE_FLOAT);
        cell.setCellValue(String.valueOf(new BigDecimal(value.toString()).setScale(2, BigDecimal.ROUND_HALF_UP)
        .doubleValue()));
    } else if(value instanceof Date) {// 如果是时间，格式yyyy-MM-dd
        cellStyle = cellStyles.get(DATATYPE_DATE);
        cell.setCellValue(DateFormatUtils.format(((Date)value),"yyyy-MM-dd"));
    } else {
        cellStyle = cellStyles.get(DATATYPE_STRING);
        cell.setCellValue((String) value);
    }
    //if(cellStyle != null) cell.setCellStyle(cellStyle);
  }

  private void export(Map<String, Object> header, List<Map<String, Object>> data, String fileName, OutputStream out) throws Exception {
    if(header == null || data == null) throw new Exception("未提供表头或数据");
    String sheetName = null;
    if(StringUtils.isNotEmpty(fileName)) {
      if(fileName.endsWith("xls") || fileName.endsWith("xlsx")) {
        sheetName = fileName.substring(0, fileName.lastIndexOf("."));
      } else {
        sheetName = fileName;
      }
    }

    HSSFWorkbook wb = new HSSFWorkbook();
    Sheet sheet = StringUtils.isEmpty(sheetName) ? wb.createSheet() : wb.createSheet(sheetName);
    sheet.setDefaultColumnWidth(16);// 设置默认列宽
    // 设置表头
    Row headerRow = sheet.createRow(0);

    Iterator<String> headers = header.keySet().iterator();
    String[] headerKeys = new String[header.size()];

    // 建立表头顺序
    int headerIndex = 0;
    while (headers.hasNext()) {
      headerKeys[headerIndex] = headers.next();
      headerIndex++;
    }

    HSSFCellStyle headerCellStyle = wb.createCellStyle();
    headerCellStyle.setVerticalAlignment(headerCellStyle.getVerticalAlignmentEnum());// 垂直
    headerCellStyle.setAlignment(headerCellStyle.getAlignmentEnum());// 水平

    HSSFFont headerFont = wb.createFont();
    headerFont.setFontHeightInPoints((short) 10);
    headerFont.setBold(true);

    headerCellStyle.setFont(headerFont);
    // 输出表头
    for (int j = 0; j < headerKeys.length; j++) {
      Cell cell = headerRow.createCell(j);
      formatExcelCell(wb, cell, header.get(headerKeys[j]));
      cell.setCellStyle(headerCellStyle);
    }

    // 输出数据
    int dataCount = data.size();
    for (int i = 0; i < dataCount; i++) {
      Row row = sheet.createRow(i + 1);
      Map<String, Object> dataRow = data.get(i);

      for (int j = 0; j < headerKeys.length; j++) {
        Cell cell = row.createCell(j);
        formatExcelCell(wb, cell, dataRow.get(headerKeys[j]));
      }
    }
    wb.write(out);
  }


  /**
   * 判断是否是整数
   * @param str
   * @return
   */
  private static boolean isInteger(String str){
    if (StringUtils.isEmpty(str)) {
      return false;
    }
    Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
    return pattern.matcher(str).matches();
  }

  /**
   * 判断是否是小数
   * @param str
   * @return
   */
  private static  boolean isDouble(String str){
    if (StringUtils.isEmpty(str)) {
      return false;
    }
    Pattern pattern = Pattern.compile("^[-\\+]?[.\\d]*$");
    return pattern.matcher(str).matches();
  }

  private   List<List<Object>>  parse(Workbook wb,int sheetIndex) throws Exception{
    List<List<Object>> data = null;
    int sheetSize = 0;
    if(wb==null || (sheetSize = wb.getNumberOfSheets()) == 0){
      return data;
    }
    Sheet sheet = wb.getSheetAt(sheetIndex);
    if(sheet==null){
      return data;
    }
    data = new ArrayList<List<Object>>();
    Row row = null;
    Cell cell = null;
    // 遍历当前sheet中的所有行
    for (int j = sheet.getFirstRowNum(); j < sheet.getLastRowNum() + 1; j++) {
      row = sheet.getRow(j);
      if (row == null || row.getFirstCellNum() == j) {
        continue;
      }
      // 遍历所有的列
      List<Object> colList = new ArrayList<Object>();
      for (int y = row.getFirstCellNum(); y < row.getLastCellNum(); y++) {
        cell = row.getCell(y);
        if (cell != null) {
          cell.setCellType(CellType.STRING);
        }
        colList.add(cell==null?"":cell.getStringCellValue());
      }
      data.add(colList);
    }
    return data;
  }


  @Override
  public void parseAndConsumer(HttpServletRequest request, HttpServletResponse response,int sheetIndex, Function function) throws Exception {
    int result = 0;//文件上传结果 0 表示上传失败，1表示 解析保存成功，-1 表示解析失败 ，2 表示无数据
    String callBack = null;
    // 获取文件流，不用保存文件
    CommonsMultipartResolver multipartResolver = new CommonsMultipartResolver(request.getSession().getServletContext());
    if (multipartResolver.isMultipart(request)) {
      MultipartHttpServletRequest multiRequest = (MultipartHttpServletRequest) request;
      //导入完毕后，调用页面的回馈方法
      callBack = multiRequest.getParameter("callBack");
      Iterator<String> it = multiRequest.getFileNames();
      while (it.hasNext()) {//一般正常情况就一个文件，所以循环也没多大实际意义
        MultipartFile file = multiRequest.getFile(it.next());
        InputStream in = new ByteArrayInputStream(file.getBytes());
        List<List<Object>>  list= parse(in,0);
        result=(int)function.apply(list);
      }
    }
    response.setContentType("text/html;charset=utf-8");
    response.getWriter().print("<script type='text/javascript'>parent." + (callBack == null ? "callBack" : callBack) + "(" + result + ");</script>");
  }

  public static void main(String[] args) throws  Exception{
    File file = new File("C:\\Users\\ZhangShuling\\Desktop","CRM接口整理2003.xls");
    //File file = new File("C:\\Users\\ZhangShuling\\Desktop","CRM接口整理.xlsx");
  //  List<List<List<Object>>> data = new ExcelServiceImpl().parse(new FileInputStream(file));
    List<List<Object>> data = new ExcelServiceImpl().parse(new FileInputStream(file),2);
    System.out.println("ok....");
  }
}
